/** 
 @file player.js
 
 @brief This file contain implementation of methods required to store data from UI.

 @author: mobile2DS

 @date: 12.1.2014
*/

var lWarn = {logln:function (o){}}; 
var lInfo = {logln:function (o){}};

var LOGFILE_NAME = "sonos-controller.log";
lWarn = new Logger(LOGFILE_NAME);
lInfo = new Logger(LOGFILE_NAME);

var Utf8 = {
    // public method for url encoding
    encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";
        for (var n = 0; n < string.length; n++) {
            var c = string.charCodeAt(n);
            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }
        }
        return utftext;
    }
};

/**
 * Store a boolean property parameter.
 * convert value if possible. In case In case conversion is not possible 
 * the value is discarded.
 * @param path [in] property path
 * @param value [in] value to store should be a boolean
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreBoolParameter(path, value) {
    try 
    {
        lInfo.logln('store boolean at |' + path + "| value: |" + value + "|" );
        var successCode = false;
        var boolValue = false;
        if (typeof(value) === "boolean") {
            Property.setProperty(path, value);
            Property.setFlag(path, 'ARCHIVE', true);       
            successCode = true;
        } else {
            if (typeof(value) === "string") {
                if (value === "true") {
                    boolValue = true;
                }
                Property.setProperty(path, boolValue);
                Property.setFlag(path, 'ARCHIVE', true);       
                successCode = true;
            } else {
                lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
            }
        }
        return successCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreBoolParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Store a string property parameter.
 * check weather the provided value is a string. In case it's a number it is converted to string, 
 * in any other cases it is discarded
 * @param path [in] property path
 * @param value [in] value to store should be string or at least a number
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreStringParameter(path, value) {
    try 
    {
        lInfo.logln('store string at |' + path + "| value: |" + value + "|" );
        var sucessCode = false;
        var stringValue = (typeof(value) === "number") ? String(value) : value;
        if (typeof(stringValue) === "string") {
            Property.setProperty(path, stringValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else {
            lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
        }
        return sucessCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreStringParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Store a integer property parameter.
 * converts provided value whenever possible to an integer. Discard the the value if this is not possible.
 * @param path [in] property path
 * @param value [in] value to store should be integer
 * @returns {Boolean} returns false if value could not be converted to integer
 */
function checkAndStoreIntParameter(path, value) {
    try 
    {
        lInfo.logln('store integer at |' + path + "| value: |" + value + "|" );
        var sucessCode = false;
        var intValue = parseInt("" + value);
        if (!isNaN(intValue)) {
            Property.setProperty(path, intValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else {
            lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
        }
        return sucessCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreIntParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Stores trigger component data to property tree
 * converts provided value whenever possible to an integer. Discard the the value if this is not possible.
 * @param idValue [in] id of activity
 * @param oParameterObject [in] trigger component data
 * @returns {Boolean} returns true if succeed
 */
function putTriggerParameter(idValue,oParameterObject) {
 var succeded = true;
 var iIndex;
    
    for (iIndex=0;iIndex<oParameterObject.triggers.length;iIndex++)
    {
      if (!oParameterObject.triggers[iIndex].bSkip) {
        var sSubpath= 'sonos-activities/'+idValue + '/triggers/' + (iIndex + 1) + "/";
        switch(oParameterObject.triggers[iIndex].type) {       
            case 'zone-scene':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene) ? succeded : false;
             if (oParameterObject.triggers[iIndex].forced!=null)
               succeded = checkAndStoreBoolParameter(sSubpath + "forced" , oParameterObject.triggers[iIndex].forced) ? succeded : false;
             
             storeSubParameterFull(sSubpath + "dsid" , -1);
                break;
            case 'undo-zone-scene':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene) ? succeded : false;
                storeSubParameterFull(sSubpath + "dsid" , -1);
                break;
            case 'device-msg':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "msg" , oParameterObject.triggers[iIndex].msg) ? succeded : false;
                if (oParameterObject.triggers[iIndex].buttonIndex!=null)
                 succeded = checkAndStoreIntParameter(sSubpath + "buttonIndex" , oParameterObject.triggers[iIndex].buttonIndex) ? succeded : false;
                break;
            case 'device-binary-input':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "index" , oParameterObject.triggers[iIndex].index) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state) ? succeded : false;
                break;
            case 'device-scene':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene) ? succeded : false;
                break;
            case 'custom-event':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreIntParameter(sSubpath + "event" , oParameterObject.triggers[iIndex].event) ? succeded : false;
                break;
            case 'device-sensor':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "eventid" , oParameterObject.triggers[iIndex].eventid) ? succeded : false;
                break;
            case 'state-change':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state) ? succeded : false;
                break;
            case 'addon-state-change':
             succeded = checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "addon-id" , oParameterObject.triggers[iIndex].addonId) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name) ? succeded : false;
             succeded = checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state) ? succeded : false;
                break;
        }
      }
    }
    return succeded;
}

/** 
 @brief Creates new ID required for storing data to PropertyTree
 
 @param sNode [in] node from whitch to get new ID
  
 @return New ID value
*/
function getNewIdValue(sNode) {
    var idValue = 0;
    var baseNode = Property.getNode(sNode);
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    if (oArrayNode != null) {
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          if (oChildNode.getChild('id') != null) {
              if (oChildNode.getChild('id').getValue() != null)
                  if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                      idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
          }
      }
    }
    return idValue;
}

/** 
 @brief Searching for player in PropertyTree
 
 @param sPlayerID [in] node from whitch to get new ID
  
 @return Id of searched player
*/
function getDefaultDeviceID(sPlayerID) {
   var idValue = -1;
   var baseNode = Property.getNode('sonos-players');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('deviceID') != null) {
           if (oChildNode.getChild('deviceID').getValue() != null && oChildNode.getChild('deviceID').getValue() == sPlayerID) {
             if (oChildNode.getChild('id').getValue() != null) {
               idValue = oChildNode.getChild('id').getValue()
               break;
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Searching for preset action in PropertyTree
 
 @param sPlayerID [in] id of player
 @param nPresetID [in] seacrhed preset ID
  
 @return Id of searched preset action, or -1 if not found
*/
function getDefaultPresetID(nRoomID, nPresetID) {
   var idValue = -1;
   var baseNode = Property.getNode('sonos-presets');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('roomID') != null) {
           if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
             if (oChildNode.getChild('presetID') != null) {
               if (oChildNode.getChild('presetID').getValue() != null && oChildNode.getChild('presetID').getValue() == nPresetID) {
                 if (oChildNode.getChild('id').getValue() != null) {
                   idValue = oChildNode.getChild('id').getValue()
                   break;
                 }
               }
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Searching dSS room from list with sonos group 
 
 @param roomsList [in] list of rooms
 @param sSonosGroup [in] sonos groupID
  
 @return Id of dSS room
*/
function getStoredRoom(roomsList, sSonosGroup) {
  var nRoomID = 0;
  for ( var iIndex = 0; iIndex < roomsList.length; iIndex++) {
    var oChildNode = roomsList[iIndex];
    if (oChildNode.groupID == sSonosGroup) {
       nRoomID = oChildNode.roomID;
       break;
    }
  }
  return nRoomID;
}

/** 
 @brief Storing Sonos Playlist to PropertyTree 
 
 @param arrayList [in] list of Sonos Playlist
  
 @return 
*/
function saveSonosPlayList(arrayList) {
    try {
        // Delete old list
        var baseNode = Property.getNode('sonos-playlists');
        if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               var oChildNode = oArrayNode[iIndex];
               baseNode.removeChild(oChildNode.getChild('id').getValue());
             }
        }
     
        if (arrayList != null) {
          for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
               var sData = arrayList[iIndex].data;      
            storeSubParameter('sonos-playlists', iIndex+'/id', iIndex);
            storeSubParameter('sonos-playlists', iIndex+'/title', unescape(arrayList[iIndex].title));
            storeSubParameter('sonos-playlists', iIndex+'/listURL', unescape(arrayList[iIndex].listCall));
            storeSubParameter('sonos-playlists', iIndex+'/playListSonosID', unescape(arrayList[iIndex].listID));
          }
        }
        
        Property.store();
    } 
    catch (e) {
        lInfo.logln("Error by saving saveSonosPlayList: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
}

/** 
 @brief Storing Sonos Favorites to PropertyTree 
 
 @param arrayList [in] list of Sonos Favorites
  
 @return 
*/
function saveSonosFavorites(arrayList) {
    try {
        // Delete old list
        var baseNode = Property.getNode('sonos-favorites');
        if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               var oChildNode = oArrayNode[iIndex];
               baseNode.removeChild(oChildNode.getChild('id').getValue());
             }
        }
     
        if (arrayList != null) {
          for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
               var sData = arrayList[iIndex].data;      
            storeSubParameter('sonos-favorites', iIndex+'/id', iIndex);
            storeSubParameter('sonos-favorites', iIndex+'/title', unescape(arrayList[iIndex].title));
            storeSubParameter('sonos-favorites', iIndex+'/description', unescape(arrayList[iIndex].description));
            storeSubParameter('sonos-favorites', iIndex+'/listURL', unescape(arrayList[iIndex].listCall));
            storeSubParameter('sonos-favorites', iIndex+'/playListSonosID', unescape(arrayList[iIndex].listID));
          }
        }
        
        Property.store();
    } 
    catch (e) {
        lInfo.logln("Error by saving saveSonosFavorites: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }                       
}

/** 
 @brief Stores Sonos Player data to PropertyTree
 
 @param objPlayer [in] - Object with Sonos player data
  
 @return 
*/ 
function storePlayer(objPlayer) {
    try {
        var playerPropertyID = getDefaultDeviceID(objPlayer.playerID);
        var bNewDevice = false; 

        if (playerPropertyID < 0) {
          playerPropertyID = getNewIdValue('sonos-players');
          bNewDevice = true;
        }
        
        // storing data to propertyTree
        storeSubParameter('sonos-players', playerPropertyID+'/id', playerPropertyID);
        storeSubParameter('sonos-players', playerPropertyID+'/ipAddress', unescape(objPlayer.deviceIP));
        storeSubParameter('sonos-players', playerPropertyID+'/sonosRoom', unescape(objPlayer.sonosRoom));
        storeSubParameter('sonos-players', playerPropertyID+'/playersList', unescape(objPlayer.playerName));
        storeSubParameter('sonos-players', playerPropertyID+'/playersListSerial', unescape(objPlayer.serialNum));
        storeSubParameter('sonos-players', playerPropertyID+'/playersListSlaves', unescape(objPlayer.playersInGroup));
        storeSubParameter('sonos-players', playerPropertyID+'/sonosGroup', unescape(objPlayer.sonosGroup));
        storeSubParameter('sonos-players', playerPropertyID+'/disabled', 'false');
        storeSubParameter('sonos-players', playerPropertyID+'/deviceID', unescape(objPlayer.playerID));
        storeSubParameter('sonos-players', playerPropertyID+'/roomID', unescape(objPlayer.roomID));
        
        Property.store();
        
        return playerPropertyID;
    } 
    catch (e) {
        lInfo.logln("Error by saving storePlayer: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
    return INVALID_PLAYER_ID;
}

/** 
 @brief Sets default empty values for action type in PropertyTree
 
 @param sPath [in] - path in PropertyTree
 @param nPropertyID [in] - id of object in PropertyTree
 @param nActionType [in] - scene action type
  
 @return 
*/ 
function setDefaultSceneAction(sPath, nPropertyID, nActionType) {
  switch(nActionType) {
    case SONOS_ACTION_NOW_PLAYING:
      storeSubParameter(sPath, nPropertyID+'/nowAction', -1);
      storeSubParameter(sPath, nPropertyID+'/nowSingleTitle', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleIsRadio', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleArtist', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleUrl', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleLength', "");
    break;
   
    case SONOS_ACTION_PLAYLIST:
      storeSubParameter(sPath, nPropertyID+'/playListTitle', "");
      storeSubParameter(sPath, nPropertyID+'/playListURL', "");
      storeSubParameter(sPath, nPropertyID+'/playListSonosID', "");
    break;

    case SONOS_ACTION_FAVORITES:
      storeSubParameter(sPath, nPropertyID+'/favoritesTitle', "");
      storeSubParameter(sPath, nPropertyID+'/favoritesURL', "");
      storeSubParameter(sPath, nPropertyID+'/favoritesSonosID', "");
    break;

    case SONOS_ACTION_COMMAND:
      storeSubParameter(sPath, nPropertyID+'/command', -1);
    break;
  }
}

/** 
 @brief Stores Preset action to PropertyTree
 
 @param objPreset [in] - Object with Preset action data
  
 @return 
*/ 
function storePreset(objPreset) {
    try {
        var nPresetID = parseInt(objPreset.triggerPreset.presetID, 10);
     
        var presetPropertyID = getDefaultPresetID(objPreset.triggerPreset.roomID, nPresetID);
        var bNewDevice = false; 

        if (presetPropertyID < 0) {
          presetPropertyID = getNewIdValue('sonos-presets');
          bNewDevice = true;
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
        }
        
        var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);
        var nRoomID = parseInt("" + (objPreset.triggerPreset.roomID), 10);
        
        // storing data to propertyTree
        storeSubParameter('sonos-presets', presetPropertyID+'/audioName', '');
        storeSubParameter('sonos-presets', presetPropertyID+'/id', presetPropertyID);
        storeSubParameter('sonos-presets', presetPropertyID+'/roomID', nRoomID);
        storeSubParameter('sonos-presets', presetPropertyID+'/presetID', nPresetID);
        storeSubParameter('sonos-presets', presetPropertyID+'/presetName', unescape(objPreset.triggerPreset.presetName));
        storeSubParameter('sonos-presets', presetPropertyID+'/volumeSet', unescape(objPreset.whatToPlay.setVolume));
        storeSubParameter('sonos-presets', presetPropertyID+'/volume', unescape(objPreset.whatToPlay.volume));         
        storeSubParameter('sonos-presets', presetPropertyID+'/actionType', nAction);
        storeSubParameter('sonos-presets', presetPropertyID+'/disabled', 'false');
                                   
        switch(nAction){                
          case 0: // now playing
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
            }

            var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
            var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
            storeSubParameter('sonos-presets', presetPropertyID+'/nowAction', nowAction);
            if (bSave) {
              if (nowAction == 0) { // single song/radio
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleIsRadio', objPreset.whatToPlay.selectedData.isRadio);
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
              }
              else { //active queue
                if (!bNewDevice) { // delete old values
                  var baseNode = Property.getNode('sonos-presets/'+presetPropertyID+'/active-queue');
                  if (baseNode != null) {
                    var oArrayNode = baseNode.getChildren();
                    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                     var oChildNode = oArrayNode[iIndex];
                      baseNode.removeChild(oChildNode.getChild('id').getValue());
                    }
                  }
                }
                
                var baseNode = Property.getNode('sonos-selected-preset');
                var queue = null;
                if (baseNode != null) {
                  queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
                  for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
                    var oQueueItem = queue[iIndex];
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/id", iIndex);
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/title", Utf8.encode(unescape(oQueueItem.title)));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/playID", unescape(oQueueItem.playID));
                    //storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/albumUri", unescape(oQueueItem.albumUri));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/artist", Utf8.encode(unescape(oQueueItem.artist)));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/trackPos", unescape(oQueueItem.trackPos));
                  }
                }
              }
            }
          break;
          
          case 1: //playlist
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
            }
    
            storeSubParameter('sonos-presets', presetPropertyID+'/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-presets', presetPropertyID+'/playListURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-presets', presetPropertyID+'/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;

          case 2: //favorites
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
            }

            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;
        }
        
        /* Storing trigger structure */
        // Delete old trigger
        var baseNode = Property.getNode('sonos-presets/'+presetPropertyID+'/triggers');
        if (baseNode != null) {
          var oArrayNode = baseNode.getChildren();
          for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
            baseNode.removeChild((iIndex+1));
          }
        }
        
        // add new trigger
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/1/type', 'zone-scene');
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/1/zone', nRoomID);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/1/group', 4);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/1/scene', nPresetID);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/1/dsid', -1);

        storeSubParameter('sonos-selected-preset', 'tmp-queue', "");

        var sPath = Property.getNode('sonos-presets/'+presetPropertyID).getPath();
        unregisterTrigger(sPath);
        registerTrigger(sPath,'sonos_execute',{});
        lInfo.logln("RegTrigger: " + sPath);
        
        storeSubParameter('sonos-presets', presetPropertyID+'/audioName', Utf8.encode(unescape(objPreset.audioName)));
        
        Property.store();
    } 
    catch (e) {
        lInfo.logln("Error by saving storePreset: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
}

/** 
 @brief Stores Activity action to PropertyTree
 
 @param objPreset [in] - Object with Activity action data
  
 @return 
*/ 
function storeActivity(objPreset) {
    try {
        var activityPropertyID = objPreset.id;
lInfo.logln("activityPropertyID="+objPreset.id);
        var bNewDevice = false; 
        if (activityPropertyID == null) {
          activityPropertyID = getNewIdValue('sonos-activities');
          bNewDevice = true;
        }
        
        if(bNewDevice) {
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
        }   
        else {
          var baseNode = Property.getNode('sonos-activities/'+activityPropertyID+'/triggers');
          if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               baseNode.removeChild((iIndex+1));
             }
          }
        }
             
        putTriggerParameter(activityPropertyID, objPreset);
        
        var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);
        
        // storing data to propertyTree
        storeSubParameter('sonos-activities', activityPropertyID+'/id', activityPropertyID);
        storeSubParameter('sonos-activities', activityPropertyID+'/volumeSet', unescape(objPreset.whatToPlay.setVolume));
        storeSubParameter('sonos-activities', activityPropertyID+'/volume', unescape(objPreset.whatToPlay.volume));        
        storeSubParameter('sonos-activities', activityPropertyID+'/setRepeat', unescape(objPreset.whatToPlay.setRepeat));
        storeSubParameter('sonos-activities', activityPropertyID+'/repeatTime', unescape(objPreset.whatToPlay.repeatTime));        
        storeSubParameter('sonos-activities', activityPropertyID+'/restartAfter', unescape(objPreset.whatToPlay.restartAfter));
        storeSubParameter('sonos-activities', activityPropertyID+'/createSonosGroup', unescape(objPreset.whatToPlay.createSonosGroup));
        storeSubParameter('sonos-activities', activityPropertyID+'/actionType', nAction);
        storeSubParameter('sonos-activities', activityPropertyID+'/disabled', 'false');
        
        switch(nAction){
          case SONOS_ACTION_NOW_PLAYING: // now playing
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }
            
            var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
            storeSubParameter('sonos-activities', activityPropertyID+'/nowAction', nowAction);
            var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
            if (bSave) {
              if (nowAction == 0) { // single song/radio
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleIsRadio', unescape(objPreset.whatToPlay.selectedData.isRadio));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
                if (objPreset.whatToPlay.selectedData.length != null && objPreset.whatToPlay.selectedData.length != "") {
                  var nLength = secondToInt(unescape(objPreset.whatToPlay.selectedData.length));
                  if (!isNaN(nLength))
                    storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleLength', nLength);
                }
              }
              else { //active queue
                if (!bNewDevice) { // delete old values
                  var baseNode = Property.getNode('sonos-activities/'+activityPropertyID+'/active-queue');
                  if (baseNode != null) {
                    var oArrayNode = baseNode.getChildren();
                    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                     var oChildNode = oArrayNode[iIndex];
                      baseNode.removeChild(oChildNode.getChild('id').getValue());
                    }
                  }
                }
                
                var baseNode = Property.getNode('sonos-selected-preset');
                var queue = null;
                if (baseNode != null) {
                  queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
                  for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
                    var oQueueItem = queue[iIndex];
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/id", iIndex);
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/title", Utf8.encode(unescape(oQueueItem.title)));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/playID", unescape(oQueueItem.playID));
                    //storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/albumUri", unescape(oQueueItem.albumUri));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/artist", Utf8.encode(unescape(oQueueItem.artist)));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/trackPos", unescape(oQueueItem.trackPos));
                  }
                }
              }
            }
          break;

          case SONOS_ACTION_PLAYLIST: //playlist
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-activities', activityPropertyID+'/playListURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-activities', activityPropertyID+'/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;

          case SONOS_ACTION_FAVORITES: //favorites
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;
          
          case SONOS_ACTION_COMMAND: // command
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/command', unescape(objPreset.whatToPlay.command));
          break;
        }
        
        // Where to play
        if (objPreset.selectedRoom.action == 0) // all rooms
          storeSubParameter('sonos-activities', activityPropertyID+'/allRooms', 1);        
        else {
          var sRooms = "";
          for ( var iIndex = 0; iIndex < objPreset.selectedRoom.rooms.length; iIndex++) {
            if (iIndex > 0) sRooms += ";"
            sRooms += objPreset.selectedRoom.rooms[iIndex];
          }
         
          storeSubParameter('sonos-activities', activityPropertyID+'/allRooms', 0);        
          storeSubParameter('sonos-activities', activityPropertyID+'/pickedRooms', sRooms);        
        }        
        
        storeSubParameter('sonos-selected-preset', 'tmp-queue', "");
         
        Property.store();
        
        var sPath = Property.getNode('sonos-activities/'+activityPropertyID).getPath();
        lInfo.logln("Registering trigger started -> "+sPath);
        unregisterTrigger(sPath);
        registerTrigger(sPath,'sonos_execute',{});
        lInfo.logln("Registering trigger finished");
    } 
    catch (e) {
        lInfo.logln("Error by saving storeActivity: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
    return false;
}

/** 
 @brief Function is checking in selected room has allready stored predefined triggers
 
 @param roomID [in] - ID of searched room
  
 @return   true if triggers are stored, false if room has no default triggers stored
*/ 
function testIfRoomHasDefaultTriggers(roomID) {
  bTestIfHasDefaultTriggers = false;
  var baseNode = Property.getNode('sonos-presets');
  if (baseNode != null) {
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
      var oChildNode = oArrayNode[iIndex];
        if (oChildNode.getChild('roomID') != null && oChildNode.getChild('roomID').getValue() == roomID) {
          if (JSON.parse(oChildNode.getChild('predefined').getValue())) {
            bTestIfHasDefaultTriggers = true;
            break;   
          }
        }
    }
  }
  return bTestIfHasDefaultTriggers;
}

/** 
 @brief Function creates predefined triggers(predefinedRoomEvents defined in file defined-values.js)
        for selected room
 @param roomID [in] - ID of room to store triggers
  
 @return  
*/ 
function createDefaultRoomPresets(roomID) {
  if (predefinedRoomEvents != null && predefinedRoomEvents.length > 0) {
    for (index in predefinedRoomEvents) {
      var presetPropertyID = getNewIdValue('sonos-presets');
      storeSubParameter('sonos-presets', presetPropertyID+'/id', presetPropertyID);
      storeSubParameter('sonos-presets', presetPropertyID+'/disabled', false);
      storeSubParameter('sonos-presets', presetPropertyID+'/predefined', true);

      storeSubParameter('sonos-presets', presetPropertyID+'/roomID', roomID);
      storeSubParameter('sonos-presets', presetPropertyID+'/presetID', -1);
      storeSubParameter('sonos-presets', presetPropertyID+'/presetName', "");
      storeSubParameter('sonos-presets', presetPropertyID+'/actionType', SONOS_ACTION_COMMAND);
      storeSubParameter('sonos-presets', presetPropertyID+'/command', index);

      storeSubParameter('sonos-presets', presetPropertyID+'/volumeSet', false);
      storeSubParameter('sonos-presets', presetPropertyID+'/volume', 0);         

      storeSubParameter('sonos-presets', presetPropertyID+'/audioName', "");
      storeSubParameter('sonos-presets', presetPropertyID+'/restartAfter', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/setRepeat', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/allRooms', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/repeatTime', 0);         
      storeSubParameter('sonos-presets', presetPropertyID+'/pickedRooms', 0);         
      
      var arrayTriggers = predefinedRoomEvents[index];
      for(trgIndx in arrayTriggers) {
        lInfo.logln("trigger"+arrayTriggers[trgIndx].id);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/type', 'zone-scene');
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/zone', roomID);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/group', arrayTriggers[trgIndx].group);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/scene', arrayTriggers[trgIndx].id);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/dsid', -1);
      }
      var sPath = Property.getNode('sonos-presets/'+presetPropertyID).getPath();
      unregisterTrigger(sPath);
      registerTrigger(sPath,'sonos_execute',{});
      lInfo.logln("RegTrigger: " + sPath);
    } 
    Property.store();
    /*{
      // add new trigger
    } */
  } 
}

/** 
 @brief Function assigns player to dss Room
 
 @param id [in] - player ID to assign room
 @param roomID [in] - ID of room 
  
 @return  
*/ 
function assignPlayerRoom(id, nRoomID) {
    try {
        var roomID = parseInt("" + unescape(nRoomID), 10);
     
        storeSubParameter('sonos-players', id+'/roomID', roomID);
        
        if (!testIfRoomHasDefaultTriggers(roomID)) {
          createDefaultRoomPresets(roomID);
        }
        
        Property.store();
    } 
    catch (e) {
        lInfo.logln("Error by saving assignPlayerRoom: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }
    }
}

/** 
 @brief Function retrives full list of sonos devices from Sonos call http://ip:port//status/topology
        and stores them as players in PT
 
 @param sSonosDeviceIP [in] - IP of sonos device
  
 @return  true if players are found and stored
*/ 
function getSonosDevices(sSonosDeviceIP) {
  var curl_options = new Object({'debug' : 0, 'connectTimeout' : URL_CONNECT_TIMEOUT});
try {        
  var h = new HTTP(curl_options, '');
  var result = h.get('http://'+sSonosDeviceIP+':'+SONOS_PORT+'/status/topology', '');
  if (result != null && result.body != null) {
    var aList = getXMLFields(result.body, 'ZonePlayer'); 
    
    var mainDevices = new Array();
    var subDevices = new Array();
    
  lInfo.logln("length="+aList.length);

    for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
      var sPlayerID = "";
      var bCoordinator = false;
      var sLocation = "";
      var sDeviceIP = ""; 
      var sPlayerSerial = "";
      var sPlayerName = "";
      var sRoomName = "";
      var sGroupID = "";

      var sData = aList[iIndex].data; 
      lInfo.logln("pos:"+iIndex);

      if (sData != null) {
        matches = sData.match(/location=\'(.*?)\'/); 
        if (matches != null) {
          sLocation = matches[1];     
     
          // check if device is speaker
          var tmpData = h.get(sLocation);
          if (tmpData != null && tmpData.body!= null && tmpData.body.indexOf("urn:schemas-upnp-org:service:MusicServices")>0) {
            var matches = sLocation.match(/:\/\/(.[^:]+)/); // get ip from full url
            if (matches != null) 
              sDeviceIP = matches[1]; 
      
            var matches = sData.match(/uuid=\'(.*?)\'/); 
            if (matches != null) 
              sPlayerID = matches[1]; 
            
            matches = tmpData.body.match(/<modelName>(.*?)<\/modelName>/);  
            if (matches != null) 
              sPlayerName = matches[1];   
                        
            matches = tmpData.body.match(/<serialNum>(.*?)<\/serialNum>/);  
            if (matches != null) 
              sPlayerSerial = matches[1];   
              
            matches = sData.match(/group=\'(.*?)\'/); 
            if (matches != null) {
              sGroupID = matches[1];     
            }

            matches = sData.match(/coordinator=\'(.*?)\'/); 
            if (matches != null) {
              bCoordinator = (matches[1].toLowerCase() == "true");     
            }
          
            matches = sData.match(/<ZonePlayer[^>]*>(.*?)<\/ZonePlayer>/); 
            if (matches != null) 
              sRoomName = matches[1];     

            if (bCoordinator) {
              mainDevices.push({playerID: sPlayerID,
                                playerName:sPlayerName, 
                                serialNum: sPlayerSerial, 
                                playersInGroup: "",
                                sonosRoom: sRoomName,
                                sonosGroup: sGroupID,
                                roomID: 0,
                                deviceIP: sDeviceIP});
            } 
            else {
              subDevices.push({playerID: sPlayerID,
                               playerName:sPlayerName, 
                               serialNum: sPlayerSerial,
                               playersInGroup: "",
                               sonosRoom: sRoomName,
                               sonosGroup: sGroupID,
                               deviceIP: sDeviceIP});
            }
              
            lInfo.logln("Found device: " + sPlayerID + " * "+sDeviceIP+" * "+sPlayerSerial+" * "+sRoomName + " * " + sGroupID);
          }
        }
      }
    } 
    
    var bDevicesFound = false;
    
    // Storing new devices to Property tree
    if (mainDevices.length > 0) {
      bDevicesFound = true;
     
      var roomsList = new Array();   
      var baseNode = Property.getNode('sonos-players');
      if (baseNode != null) {
        var oArrayNode = baseNode.getChildren();
        for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          lInfo.logln("player:"+iIndex);
          var oChildNode = oArrayNode[iIndex];
            if (oChildNode.getChild('sonosGroup') != null && oChildNode.getChild('sonosGroup').getValue() != null) 
                if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() > 0)  {
                  roomsList.push({
                                     groupID:oChildNode.getChild('sonosGroup').getValue(), 
                                     roomID:oChildNode.getChild('roomID').getValue()
                                    });
                }
        }
      }
        
      // Delete old devices
      var baseNode = Property.getNode('sonos-players');
      if (baseNode != null) {
           var oArrayNode = baseNode.getChildren();
           for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
             var oChildNode = oArrayNode[iIndex];
             baseNode.removeChild(oChildNode.getChild('id').getValue());
           }
      }
      
      for ( var iIndex = 0; iIndex < mainDevices.length; iIndex++) {
        // search if has any sub devices
        var sSubDevices = ""; 
        var sSubDevicesSer = ""; 
        var sSubDevicesGroup = "";
        var sSubRooms = ""; 
        for ( var yIndex = 0; yIndex < subDevices.length; yIndex++) {
           if (subDevices[yIndex].sonosGroup == mainDevices[iIndex].sonosGroup) {
             if (sSubDevices.length > 0) 
               sSubDevices = sSubDevices + ", ";
             sSubDevices = sSubDevices + subDevices[yIndex].playerName;

             if (sSubDevicesSer.length > 0) 
               sSubDevicesSer = sSubDevicesSer + ", ";
             sSubDevicesSer = sSubDevicesSer + subDevices[yIndex].serialNum;

             if (sSubDevicesGroup.length > 0) 
               sSubDevicesGroup = sSubDevicesGroup + ", ";
             sSubDevicesGroup = sSubDevicesGroup + subDevices[yIndex].playerID;
             
             if (subDevices[yIndex].sonosRoom != mainDevices[iIndex].sonosRoom) {
               if (sSubRooms.length > 0) 
                 sSubRooms = sSubRooms + ", ";
               sSubRooms = sSubRooms + subDevices[yIndex].sonosRoom;
             }
           }
        }
        
        if (sSubDevices.length > 0)  {
          mainDevices[iIndex].playerName = mainDevices[iIndex].playerName + ", " + sSubDevices;
        }

        if (sSubDevicesSer.length > 0)  {
          mainDevices[iIndex].serialNum = mainDevices[iIndex].serialNum + ", " + sSubDevicesSer;
        }

        mainDevices[iIndex].playersInGroup = "";
        if (sSubDevicesGroup.length > 0)  {
          mainDevices[iIndex].playersInGroup = sSubDevicesGroup;
        }

        
        if (sSubRooms.length > 0)  {
          mainDevices[iIndex].sonosRoom = "Grouped ["+mainDevices[iIndex].sonosRoom + ", " + sSubRooms+"]";
        }
        
        lInfo.logln("Search group "+mainDevices[iIndex].sonosGroup)
        mainDevices[iIndex].roomID = getStoredRoom(roomsList, mainDevices[iIndex].sonosGroup);
        
        var playerID = storePlayer(mainDevices[iIndex]);
        if (playerID != INVALID_PLAYER_ID)
          createStates(playerID, 0);
      } 
    }
    
    return (mainDevices.length > 0);
  }
 }
 catch (e) {
      lInfo.logln("Error executing getSonosDevices " + e);
 }
 return false;
}

/** 
 @brief Function creates all triggers for Preset and Activities action
 
 @param -
  
 @return  
*/ 
function updateAllTriggers(){
    var baseNode = Property.getNode('sonos-presets');
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        var sPath = oChildNode.getPath();
        lInfo.logln("sonos-presets Create Trigger["+iIndex+"] " + sPath);
        registerTrigger(oChildNode.getPath(),'sonos_execute',{});
    }

    var baseNode = Property.getNode('sonos-activities');
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        var sPath = oChildNode.getPath();
        lInfo.logln("sonos-activities Create Trigger["+iIndex+"] " + sPath);
        registerTrigger(oChildNode.getPath(),'sonos_execute',{});
    }
}

/** 
 @brief Function creates predefined triggers(predefinedStopActivities defined in file defined-values.js)
        for all aactivities
        
 @param roomID [in] - ID of room to store triggers
  
 @return  
*/ 
function preparePredefinedActivities() {
    // test if predefined activities are created
    var arrayStored = new Array();
    arrayStored['zone-scene'] = new Array();
    var baseNode = Property.getNode('sonos-activities');
    if (baseNode != null) {
      var oArrayNode = baseNode.getChildren();
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        var oTriggers = oChildNode.getChild('triggers').getChildren();
        if (oTriggers != null) {
          for ( var yIndex = 0; yIndex < oTriggers.length; yIndex++) {
             var oChildTrigger = oTriggers[yIndex];
             var nType = oChildTrigger.getChild('type').getValue();
             switch(nType) {
                case 'zone-scene':
                  arrayStored['zone-scene'].push(oChildTrigger.getChild('scene').getValue()); 
                break;
             }
          }
        }
      }
    }
 
    for (var yIndex=0;yIndex<predefinedStopActivities.length;yIndex++) {
      var command = predefinedStopActivities[yIndex];
      var bStore = true;
    
      switch(command.type) {       
          case 'zone-scene':
            if (arrayStored['zone-scene'].indexOf(command.scene) > -1)
              bStore = false;
          break;
      }
    }
    if (bStore) {
      // Store predefined Activity
      var objActivity = new Object();
      objActivity.id = null;
      objActivity.triggers = new Array();
      objActivity.triggers.push(command);
      objActivity.whatToPlay = new Object();
      objActivity.whatToPlay.actionType = SONOS_ACTION_COMMAND;
      objActivity.whatToPlay.command = COMMAND_STOP;
      objActivity.whatToPlay.setVolume = 0;
      objActivity.whatToPlay.volume = 0;
      objActivity.whatToPlay.setRepeat = 0;
      objActivity.whatToPlay.repeatTime = 0;        
      objActivity.whatToPlay.restartAfter = 0;
      objActivity.whatToPlay.createSonosGroup = 0;
      objActivity.selectedRoom = new Object();
      objActivity.selectedRoom.action = 0; //all rooms
      
      storeActivity(objActivity);
    } 
}

if (raisedEvent.name === 'running') {
    lInfo.logln("running sonos players.js");
    Property.load();
    lInfo.logln("running sonos players.js end");
}
else
  if (raisedEvent.name == 'model_ready') {
    lInfo.logln("model_ready sonos players.js");
  
    updateAllTriggers();  
    
    preparePredefinedActivities();
    
    storeSubParameter('sonos-grouping', 'data', '');
    storeSubParameter('sonos-grouping', 'sceneID', 0);
    Property.store();
    lInfo.logln("model_ready sonos players.js end");
  }
else
  if (raisedEvent.name == 'sonos-controller.save') {
    if (raisedEvent.parameter.actions == 'assignRoom') {
       var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
       lInfo.logln("Assign room:"+sObject.playersList.roomID); 
       if (sObject != null && sObject.playersList.roomID > 0)
         assignPlayerRoom(sObject.id, sObject.playersList.roomID);

       var evt = new Event("sonos-controller-saved",  {});
       evt.raise();
    }
    if (raisedEvent.parameter.actions == 'deletePreset') {
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));

        if (typeof(oIDObject)=='number') {
          oIDObject=[oIDObject];
        }
        
        for (var sKey in oIDObject) {
          var baseNode = Property.getNode("sonos-presets");
          if (baseNode != null) {
             var sPath = Property.getNode('sonos-presets/'+oIDObject[sKey]).getPath();
             lInfo.logln("Delete trigger " + sPath);
             unregisterTrigger(sPath);
             lInfo.logln("Delete trigger finished");

             lInfo.logln("Delete child " + oIDObject[sKey]);
             baseNode.removeChild(oIDObject[sKey]);
          }
        }
        Property.store();
        
        var evt = new Event("sonos-controller-saved",  {});
        evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'deleteActivity') {
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        
        if (typeof(oIDObject)=='number') {
          oIDObject=[oIDObject];
        }
        
        for (var sKey in oIDObject) {
          var baseNode = Property.getNode("sonos-activities");
          if (baseNode != null) {
             var sPath = Property.getNode('sonos-activities/'+oIDObject[sKey]).getPath();
             lInfo.logln("Delete trigger " + sPath);
             unregisterTrigger(sPath);
             lInfo.logln("Delete trigger finished");

             lInfo.logln("Delete child " + oIDObject[sKey]);
             baseNode.removeChild(oIDObject[sKey]);
          }
        }
        Property.store();
          
        var evt = new Event("sonos-controller-saved",  {});
        evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'storePreset') {
       var objData = JSON.parse(unescape(raisedEvent.parameter.value));
       if (objData != null) {
         lInfo.logln("Store preset"); 
         storePreset(objData);
       }
       var evt = new Event("sonos-controller-saved",  {});
       evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'storeActivity') {
       var objData = JSON.parse(unescape(raisedEvent.parameter.value));
       if (objData != null) {
         lInfo.logln("Store activity"); 
         var bSaved = storeActivity(objData);
       }
      lInfo.logln("Store activity create saved event");  
       var evt = new Event("sonos-controller-saved", {value:(JSON.stringify(bSaved))});
       evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'enablePreset') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPresetID = parseInt("" + oIDObject[sKey],10);
        if (nPresetID > -1) {  
         storeSubParameter('sonos-presets', nPresetID+'/disabled', false);
         var sPath = Property.getNode('sonos-presets/'+nPresetID).getPath();
         registerTrigger(sPath,'sonos_execute',{});
lInfo.logln("Enable Preset "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disablePreset') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPresetID = parseInt("" + oIDObject[sKey],10);
        if (nPresetID > -1) {  
         storeSubParameter('sonos-presets', nPresetID+'/disabled', true);
         var sPath = Property.getNode('sonos-presets/'+nPresetID).getPath();
         unregisterTrigger(sPath);
lInfo.logln("Disabled Preset "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
    else
    if (raisedEvent.parameter.actions == 'enableActivity') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nActivityID = parseInt("" + oIDObject[sKey],10);
        if (nActivityID > -1) {  
         storeSubParameter('sonos-activities', nActivityID+'/disabled', false);
         var sPath = Property.getNode('sonos-activities/'+nActivityID).getPath();
         registerTrigger(sPath,'sonos_execute',{});
lInfo.logln("Enable Activity "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disableActivity') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nActivityID = parseInt("" + oIDObject[sKey],10);
        if (nActivityID > -1) {  
         storeSubParameter('sonos-activities', nActivityID+'/disabled', true);
         var sPath = Property.getNode('sonos-activities/'+nActivityID).getPath();
         unregisterTrigger(sPath);
lInfo.logln("Disabled Activity "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
    else
    if (raisedEvent.parameter.actions == 'enablePlayer') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPlayerID = parseInt("" + oIDObject[sKey],10);
        if (nPlayerID > -1) {  
         storeSubParameter('sonos-players', nPlayerID+'/disabled', false);
lInfo.logln("Enabled Player "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disablePlayer') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPlayerID = parseInt("" + oIDObject[sKey],10);
        if (nPlayerID > -1) {  
         storeSubParameter('sonos-players', nPlayerID+'/disabled', true);
lInfo.logln("Disabled Player "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
  }
  else
    if (raisedEvent.name == 'sonos-controller.startdiscovery') {
      if (raisedEvent.parameter.actions == 'discover') {
      var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
lInfo.logln("startdiscovery="+sObject.discoverIP);
      var sResult = "";
      if (getSonosDevices(sObject.discoverIP)) {
        sResult = sObject.discoverIP;

        preparePredefinedActivities();
        
        storeSubParameter('sonos-grouping', 'data', '');
        storeSubParameter('sonos-grouping', 'sceneID', 0);
      }
      
      var evt = new Event("sonos-discover-finished",  {ip:sResult});
      evt.raise(); 
      lInfo.logln("End discovery");
    }
    else if (raisedEvent.parameter.actions == 'AutoDiscover') {
        var arrayAdresses = JSON.parse(unescape(raisedEvent.parameter.value));
        var curl_options = new Object({
          'debug' : 0,
          'connectTimeout' : URL_CONNECT_TIMEOUT
        });
 lInfo.logln("AutoDiscover "+unescape(raisedEvent.parameter.value));        
        var h = new HTTP(curl_options, '');
        for ( var iIndex = 0; iIndex < arrayAdresses.length; iIndex++) {
          var oChild = arrayAdresses[iIndex];
          var sIP = oChild.ipAddress;
          
          if (oChild.inList == 1)  { // testing on existing IP from grid
            lInfo.logln("Autodiscovery on existing IP "+sIP);
           
            var sUrl = "http://"+sIP+":"+SONOS_PORT+"/xml/device_description.xml";
            lInfo.logln("Testurl "+sUrl);

            var result = h.get(sUrl);
            if (result != null && result.body != null)
            {
              lInfo.logln("Result: IP returns result");
              if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
                lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
                if (getSonosDevices(sIP)) {
                  lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
                  sResult = sIP;
                  lInfo.logln("Create predefined activities");
                  preparePredefinedActivities();

                  break;          
                }
              }
            }
          }
          else {  // testing dSS IP and range 1-255
            var aIP = sIP.split('.');
            if (aIP.length == 4)
              sIP = aIP[0]+"."+aIP[1]+"."+aIP[2]+".";
            
            lInfo.logln("Testing dSS-IP range "+sIP);
          
            var sResult = "";
            for ( var iIndex = 1; iIndex < 256; iIndex++) {
              var sSearchedIP = sIP+iIndex;
             
              var sUrl = "http://"+sSearchedIP+":"+SONOS_PORT+"/xml/device_description.xml";
              lInfo.logln("Testurl "+sUrl);

              var result = h.get(sUrl);
              if (result != null && result.body != null)
              {
                lInfo.logln("Result: IP returns result");
                if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
                  lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
                  if (getSonosDevices(sSearchedIP)) {
                    lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
                    sResult = sIP;
                    lInfo.logln("Create predefined activities");
                    preparePredefinedActivities();
                    
                    storeSubParameter('sonos-grouping', 'data', '');
                    storeSubParameter('sonos-grouping', 'sceneID', 0);
                    
                    break;          
                  }
                }
              }
              result = null;
              lInfo.logln("----------");
            }
          }

          lInfo.logln("IP:"+ oChild.ipAddress + " / inList:" + oChild.inList);
        }
        
        var evt = new Event("sonos-autodiscover-finished",  {ip:sResult});
        evt.raise(); 
    }
  }
  else
    if (raisedEvent.name == 'sonos-controller.getdata') {
         if (raisedEvent.parameter.actions == 'getSonosQueue') { 
           lInfo.logln("Inside getSonosQueue");

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000 /* in mseconds */
            });
             
           var arrayLists = upnpGetCurrentQueue(oObject, 100, curl_options);
           
           var data = JSON.stringify(arrayLists);     
           lInfo.logln(data);
           storeSubParameter('sonos-selected-preset', 'tmp-queue', data);
           Property.store();
           
           
           var evt = new Event("sonos-discover-return-sonos-queue", {returnValue : data});
           evt.raise();
         }
         else
         if (raisedEvent.parameter.actions == 'getSonosTrack') { 

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
           lInfo.logln("Inside getSonosTrack for IP:"+oObject);
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000 /* in mseconds */
            });
                                                       
           var objCurrentTrack = upnpGetCurrentTrack(oObject, curl_options);

           var data = JSON.stringify(objCurrentTrack);
           var evt = new Event("sonos-discover-return-sonos-track", {returnValue : data});
           evt.raise();
         }
         else
         if (raisedEvent.parameter.actions == 'getSonosPlaylists') { 
           lInfo.logln("Inside getSonosPlaylists");

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000 /* in mseconds */
            });
                                                       
           var arrayLists = upnpGetSonosPlaylist(oObject, 100,curl_options);
           saveSonosPlayList(arrayLists);
           
           arrayLists = upnpGetSonosFavorites(oObject, 100, curl_options);
           saveSonosFavorites(arrayLists);
         }
         if (raisedEvent.parameter.actions == 'getCurrentPresetRoom') { 
           lInfo.logln("Inside getCurrentPresetRoom");
                            
           var baseNode = Property.getNode('sonos-selected-preset');
           var roomID = -1;
           if (baseNode != null) {
             roomID = baseNode.getChild('roomID').getValue();
           }
                            
           var data = escape(JSON.stringify(objCurrentTrack));
           var evt = new Event("sonos-return-preset-room", {returnValue : roomID});
           evt.raise();
         }

    }


