Ext.define('DSS.component.SonosRoomLabel', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
                 return 20;
    },
    lblLabel:null,
    constructor : function(config) {

        this.lblLabel  = Ext.create('Ext.form.field.Display',{
          fieldLabel: _('Sonos Room'),
        }); 
        
        Ext.apply(this, {
            items : [ this.lblLabel ]
        });                          
        this.callParent(arguments);
    },
    reset: function (){
     this.lblLabel.setValue(' - ');
    },
    setValue:function (value){
      this.lblLabel.setValue(unescape(value));
    },
    isValid:function(){
        return true;
    },
    getValue : function(){

    }
});

Ext.define('DSS.component.SonosDeviceLabel', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
       return 200;
    },
    lblLabel:null,
    gridRooms:null,
    roomID:-1,
    roomRenderer : function(val, metadata, record, colIdx, store, view) {
       var sPassive="";        
       if (record.get('passive')!=null && record.get('passive'))
         sPassive='style="color:#888888;"';
       var sReturn='<table style="height:16px;"><tr>';
       sReturn+='<td '+sPassive+' width="100%">' + record.get('name') + '</td>';
       return sReturn +'</tr></table>';      
    },
    initStores : function() {
       var sString = dss.ajaxSyncRequest('/json/property/query', {
          'query' : '/scripts/sonos-controller/sonos-players/*(*)/*(*)/*(*)/*(*)'
           });
       var result = Ext.JSON.decode(sString);
       var arrayActiveRooms = dss.parsePropertyQueryAnswer(result,[{name:'id', type:'simple'},
                                                                   {name:'roomID',type:'simple'}]);    
       
       var arrayRooms = new Array();                                                             
       if (dss.staticDataModel.nameCache != null) 
       {
         for ( var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
           var oZone = dss.staticDataModel.nameCache[iIndex];
           if (oZone.ZoneID > 0) {
             var bPassive = false;
             for ( var yIndex = 0; yIndex < arrayActiveRooms.length; yIndex++) {
               if (oZone.ZoneID == arrayActiveRooms[yIndex].roomID && this.roomID != oZone.ZoneID) {
                 bPassive = true;
                 break;
               }
             }
            
             arrayRooms.push({"_id":oZone.ZoneID,
                              "name":oZone.name,
                              "passive":bPassive});
           }
         } 
       } 
                                                                          
       Ext.create('Ext.data.Store', {
                storeId : 'assign_sonos_players',
                fields : [ '_id', 'name', 'passive'],
                proxy : {
                    type : 'memory',
                    reader : {
                        type : 'json',
                        root : 'items'
                    }
                }
       });
       Ext.data.StoreManager.lookup('assign_sonos_players').add(arrayRooms);
    },
    constructor : function(config) {
        this.lblLabel  = Ext.create('Ext.form.field.Display',{
          fieldLabel: _('Devices'),
        }); 
                     
        var lblRoom  = Ext.create('Ext.form.field.Display',{
          fieldLabel: _('Room'),
          margin : '0 0 0 0',
        }); 

        this.gridRooms = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            autoScroll : true,
            width:200,
            height:150,
            margin : '5 0 0 0',
            viewConfig : {
                stripeRows : true
            },
            selModel : {
                mode : 'SINGLE'
            },
            columns : [ {
                sortable : false,
                hideable : false,
                dataIndex : '_id',
                flex : 1,
                renderer : this.roomRenderer
            } ]
        }); 
        
        this.gridRooms.on('itemclick', function(grid, record) {
            var fIsPassive=false;
            if (record.get('passive')!=null) {
                if (record.get('passive')==true) {
                    this.getSelectionModel().deselect(record);
                    fIsPassive=true;
                }
            }
        }); 
        
        var pnlRoom = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [ lblRoom, this.gridRooms ],
            layout: {type: 'hbox'},
            margin : '5 0 5 0',
            width : 400 });
        
        Ext.apply(this, {
            items : [this.lblLabel, pnlRoom]//, 
        });                          
        this.callParent(arguments);
    },
    reset: function (){
      this.lblLabel.setValue(' - ');
      this.initStores();
      this.gridRooms.reconfigure(Ext.data.StoreManager.lookup('assign_sonos_players'));
      this.roomID = -1;
    },
    setValue:function (value){
      this.roomID = parseInt("" + this.dssEditor.editControls.roomID.getValue(),10);
      this.lblLabel.setValue(unescape(value));
      this.initStores();
      this.gridRooms.reconfigure(Ext.data.StoreManager.lookup('assign_sonos_players'));
      var store = this.gridRooms.getStore();
      var foundRec = store.getAt(store.findExact('_id', this.roomID));
      this.gridRooms.getSelectionModel().select(foundRec);
    },
    isValid:function(){
       var oSelectionRooms = this.gridRooms.getSelectionModel().getSelection();
       var bOk = (oSelectionRooms.length > 0);
       if (!bOk)
         Ext.MessageBox.alert(_("Info"), _("Please select room from list"), null);
       return bOk;
    },
    getValue : function(){
      var returnValue = new Object();
      
      var oSelectionRooms = this.gridRooms.getSelectionModel().getSelection();
      if (oSelectionRooms.length > 0) {
          returnValue.roomID = parseInt("" + oSelectionRooms[0].get('_id'), 10);
      }
      return returnValue;      
    }
});